from datetime import datetime, date, timedelta
from logging import getLogger

from django.db import models
from django.conf import settings

from .constants import colors

User = settings.AUTH_USER_MODEL
logger = getLogger(__name__)


class KakeboMonth(models.Model):
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    month = models.PositiveSmallIntegerField(default=1)
    year = models.PositiveSmallIntegerField(default=1980)
    budget: dict = models.JSONField(default=dict)
    spare_cost = models.TextField(blank=True, null=True, default="")
    target_reach = models.TextField(blank=True, null=True, default="")
    spare = models.FloatField(blank=True, null=True, default=0)

    class Meta:
        verbose_name_plural = "Kakebo Month"

    def __str__(self):
        return f"{self.user.username}'s Kakebo - {self.month}/{self.year}"

    @property
    def display_totals_budget(self) -> (float, float):
        income = "%.2f" % self.get_totals_budget[0]
        outflow = "%.2f" % self.get_totals_budget[1]
        return income, outflow

    @property
    def display_available_money(self):
        income, outflow = self.display_totals_budget
        if self.spare:
            return float(income) - float(outflow) - self.spare
        return None

    @property
    def get_totals_budget(self) -> (float, float):
        income = float(0)
        outflow = float(0)
        for key, val in self.budget.items():
            if "_income_" in key:
                value = val.get("value_income") if val.get("value_income") else 0
                income += float(value)
            if "_outflow_" in key:
                value = val.get("value_outflow") if val.get("value_outflow") else 0
                outflow += float(value)
        return income, outflow


class KakeboWeek(models.Model):
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    month = models.ForeignKey(KakeboMonth, on_delete=models.CASCADE)
    week = models.IntegerField(default=1)

    class Meta:
        verbose_name_plural = "Kakebo Week"

    def __str__(self):
        start = self.display_start_week
        end = self.display_end_week
        return f"{self.user.username}'s Kakebo - {start} // {end}"

    @property
    def display_start_week(self) -> date:
        date_w = f"{self.month.year}-{self.week}-1"
        return datetime.strptime(date_w, "%Y-%W-%w").date()

    @property
    def display_end_week(self) -> date:
        return self.display_start_week + timedelta(days=6)

    @property
    def total_week(self):
        return sum([obj.display_total_table for obj in KakeboWeekTable.objects.filter(kakebo=self)])


class KakeboWeekTable(models.Model):
    kakebo = models.ForeignKey(KakeboWeek, on_delete=models.CASCADE)
    data_row: dict = models.JSONField(default=dict)

    LIST_TYPE = [
        (0, "basic necessities"),
        (1, "optional"),
        (2, "culture and leisure"),
        (3, "extras and unexpected"),
    ]
    type_cost: int = models.PositiveSmallIntegerField(
        choices=LIST_TYPE,
        default=0,
    )

    @property
    def display_type_cost_color(self) -> str:
        return colors[self.type_cost]

    @property
    def display_total_table(self) -> float:
        return sum([self.total_column(i) for i in range(len(self.data_row))])

    @property
    def list_sort_cost(self) -> list:
        data = []
        for row in self.data_row.values():
            data.extend([v for v in row.values()])
        sort_ = sorted([x for x in data], key=lambda x: x['value'], reverse=True)
        return sort_

    def get_column(self, clm: int) -> list:
        clm = self.data_row.get(f'{clm}', {})
        if clm:
            return [(x.get('desc', ""), x.get('value', 0)) for x in clm.values()]
        return []

    def get_cell(self, clm: int, row: int) -> tuple:
        return self.get_column(clm)[row]

    def get_list_sort_cost(self, max_row: int) -> list:
        max_row -= 1
        list_cost = self.list_sort_cost[:max_row]

        if list_cost:
            other_cost = 0
            for i in self.list_sort_cost[max_row:]:
                other_cost += i["value"]
            if len(list_cost) >= max_row:
                list_cost.append({"desc": "others", "value": other_cost})
        return list_cost

    def total_column(self, clm: int) -> float:
        clm = self.get_column(clm)
        if clm:
            return sum([x[1] for x in clm])
        return float(0)


class KakeboEndOfMonthBalance(models.Model):
    month = models.ForeignKey(KakeboMonth, on_delete=models.CASCADE)
    electricity = models.FloatField(default=0)
    gas = models.FloatField(default=0)
    tel_internet = models.FloatField(default=0)
    water = models.FloatField(default=0)
    waste = models.FloatField(default=0)
    costs_data: dict = models.JSONField(default=dict)
    answer_1 = models.TextField(blank=True, null=True, default="")
    answer_2 = models.TextField(blank=True, null=True, default="")
    answer_3 = models.TextField(blank=True, null=True, default="")

    LIST_TYPE = [
        (0, ""),
        (1, "yes"),
        (2, "almost"),
        (3, "no"),
    ]
    conclusion: int = models.PositiveSmallIntegerField(
        choices=LIST_TYPE, default=0,
    )

    @property
    def display_electricity(self):
        return "%.2f" % self.electricity

    @property
    def display_gas(self):
        return "%.2f" % self.gas

    @property
    def display_tel_internet(self):
        return "%.2f" % self.tel_internet

    @property
    def display_water(self):
        return "%.2f" % self.water

    @property
    def display_waste(self):
        return "%.2f" % self.waste

    @property
    def display_total_utilities(self):
        return "%.2f" % self.total_utilities

    @property
    def total_utilities(self):
        attrs = ["electricity", "gas", "tel_internet", "water", "waste"]
        return sum([getattr(self, x) for x in attrs])

    @property
    def display_diff_available_costs(self):
        return "%.2f" % self.diff_available_costs

    @property
    def display_total_month(self):
        return "%.2f" % self.total_month

    @property
    def total_month(self):
        val = sum([x.total_week for x in KakeboWeek.objects.filter(month=self.month)])
        val += self.total_utilities
        return val

    @property
    def diff_available_costs(self):
        return self.month.display_available_money - self.total_month

    def get_key_list(self):
        """
        get a list of all key in costs_data
        """
        return list(self.costs_data.keys())

    def get_key_cost(self, nr_key: int = 0) -> str:
        key_list = self.get_key_list()
        if nr_key <= len(key_list) - 1:
            key = key_list[nr_key]
        else:
            key = ""
        return key

    def tot_costs(self, nr_key: int, counter0: bool = True) -> float:
        """
        :counter0:  True if the counter starts from zero, if not put it false
        """
        if not counter0:
            nr_key -= 1
        key = self.get_key_cost(nr_key)
        val = 0
        if key:
            val = sum([x for x in self.costs_data[key].values() if x])
        return val


# settings models
class UtilitiesCost(models.Model):
    name = models.CharField(max_length=150)



from django.contrib import admin

from .models import KakeboMonth, KakeboWeek, KakeboWeekTable, KakeboEndOfMonthBalance


@admin.register(KakeboMonth)
class KakeboMonthAdmin(admin.ModelAdmin):
    pass


@admin.register(KakeboWeek)
class KakeboWeekAdmin(admin.ModelAdmin):
    search_fields = ['kakebo']
    autocomplete_fields = ['user']


@admin.register(KakeboWeekTable)
class KakeboWeekTableAdmin(admin.ModelAdmin):
    pass


@admin.register(KakeboEndOfMonthBalance)
class KakeboEndOfMonthBalanceAdmin(admin.ModelAdmin):
    pass



from django.urls import path

from .views.kakebo import Index, SelectYearWeekFormView, KakeboWeekFormView, EndOfMonthBalanceSheetFormView

urlpatterns = [
    path("", Index.as_view(), name="kakebo-home"),
    path("calendar/", SelectYearWeekFormView.as_view(), name="kakebo-calendar"),
    path("calendar/<int:year>/", SelectYearWeekFormView.as_view(), name="kakebo-calendar"),
    path("calendar/<int:year>/<int:month>/", SelectYearWeekFormView.as_view(), name="kakebo-calendar"),
    path("<int:year>/<int:week>/", KakeboWeekFormView.as_view(), name="kakebo-week"),
    path("calendar/<int:year>/<int:month>/report/", EndOfMonthBalanceSheetFormView.as_view(), name="kakebo-balance"),
]



from django.views.generic import TemplateView
from django.contrib.auth.mixins import LoginRequiredMixin


class KakeboDashboardSetting(LoginRequiredMixin, TemplateView):
    template_name = "settings/dashboard.html"


class KakeboSettings(LoginRequiredMixin, TemplateView):
    template_name = "settings/settings.html"


class KakeboThemes(LoginRequiredMixin, TemplateView):
    template_name = "settings/themes.html"



from django.urls import path, include
from django.contrib.auth import views as auth_view

from .views import (
    RegisterPageFormView,
    # active_user_mail,
    AccountLoginView,
    AccountPasswordResetView,
    AccountPasswordResetDoneView,
    AccountPasswordResetConfirmView,
    AccountPasswordResetCompleteView,
)

urlpatterns = []

urlpatterns += [
    path(
        "sign_up/",
        RegisterPageFormView.as_view(),
        name="sign_up",
    ),
    # path(
    #     "active/<str:uidb64>/<str:token>",
    #     active_user_mail,
    #     name="active-user-mail",
    # ),
    path(
        "login/",
        AccountLoginView.as_view(),
        name="login",
    ),
    path(
        "logout/",
        auth_view.LogoutView.as_view(
            template_name="user/sign-out.html",
        ),
        name="logout",
    ),
    path(
        "password/change/",
        auth_view.PasswordChangeView.as_view(
            template_name="user/password-change.html",
        ),
        name="password_change",
    ),
    path(
        "password/change/conferma",
        auth_view.PasswordChangeDoneView.as_view(
            template_name="user/password-change-done.html",
        ),
        name="password_change_done",
    ),
    path(
        "password/reset/",
        AccountPasswordResetView.as_view(),
        name="password_change_done",
    ),
    path(
        "password/reset/thanks/",
        AccountPasswordResetDoneView.as_view(),
        name="reset_password",
    ),
    path(
        "password/reset/confirm/<uidb64>/<token>/",
        AccountPasswordResetConfirmView.as_view(),
        name="reset_password_done",
    ),
    path(
        "password/reset/complete/",
        AccountPasswordResetCompleteView.as_view(),
        name="reset_password_confirm",
    ),
]

urlpatterns += [
    path("", include("django.contrib.auth.urls")),
]



from django import template
from django.contrib.auth.models import User
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _
from ..constants import colors as list_colors, name_type_cost
from ..models import KakeboMonth, KakeboWeek, KakeboWeekTable, KakeboEndOfMonthBalance

register = template.Library()


@register.filter(is_safe=True)
@register.simple_tag
def render_total_weeks(kakebo: str, color: str):
    username, year, month = kakebo.split('-')
    # get User()
    user = User.objects.get(username=username)
    # get KakeboMonth()
    obj_month = KakeboMonth.objects.get(
        user=user,
        month=month,
        year=year,
    )
    obj_list = KakeboWeek.objects.filter(user=user, month=obj_month)
    type_cost = int(list_colors.index(color))
    data = []
    for obj in obj_list:
        val = KakeboWeekTable.objects.get(kakebo=obj, type_cost=type_cost).display_total_table
        data.append("%.2f" % val)

    html = f"""
        <table class="w-100">
            <thead>
                <tr>
                    <th colspan="2">
                        <p class="bg-{color} py-2 ps-3 text-white">
                            {name_type_cost[type_cost]}
                        </p>
                    </th>
                </tr>
            </thead>
            <tbody>
    """

    for i, val in enumerate(data, 1):
        html += f"""
            <tr>
                <td class="bx-{color} px-15">
                    <p class="mb-2">
                        week {i}
                    </p>
                </td>
                <td class="px-15">
                    <p class="bb-dashed-{color} mb-2 text-end">
                        € {val}
                    </p>
                </td>
            </tr>
        """

    total = "%.2f" % sum(list(map(float, data)))
    html += f"""
            <tr>
                <td class="bg-{color} bx-{color} px-15">
                    <p class="mt-2 mb-1 text-white">
    """
    html += _("total")
    html += f"""
                    </p>
                </td>
                <td class="px-15">
                    <p class="bb-dashed-{color} mb-2 text-end">
                        € {total}
                    </p>
                </td>
            </tr>
        """

    html += """
        </tbody>
    </table>
    """
    return mark_safe(html)


@register.filter(is_safe=True)
@register.simple_tag
def render_cost_relevant(kakebo: str, nr_cost: int, row: int = 5, disabled: bool = False):
    disabled = "disabled" if disabled else ""
    username, year, month = kakebo.split('-')
    # get User()
    user = User.objects.get(username=username)
    # get KakeboMonth()
    obj_month = KakeboMonth.objects.get(
        user=user,
        month=month,
        year=year,
    )
    obj = KakeboEndOfMonthBalance.objects.get(month=obj_month)
    costs = obj.costs_data

    key, data = "", {}
    if costs:
        key_list = list(costs.keys())
        if len(key_list) >= nr_cost:
            key = key_list[nr_cost - 1]
            data = costs[key]

    html = f"""
    <table class="w-100">
        <thead>
            <tr>
                <th>
                    <p class="bg-slategrey py-2 ps-3 text-white">
                        costo {nr_cost}
                    </p>
                </th>
                <th class="px-15">
                    <p class="bb-dashed-slategrey mb-2 text-end">
                        <input type="text" name="cost_{nr_cost}_name" id="id_cost_{nr_cost}_name" {disabled}
    """

    if key:
        html += f"value='{key}' class='text-end'>"
    else:
        html += f">"

    html += f"""
                    </p>
                </th>
            </tr>
        </thead>
        <tbody>
    """

    for i in range(1, row + 1):
        html += f"""
            <tr>
                <td class="bs-slategrey px-15">
                    <p class="mb-2">
                        week {i}
                    </p>
                </td>
                <td class="px-15">
                    <p class="bb-dashed-slategrey mb-2 text-end">
                        <input type="number" step="any" min="0"  placeholder="0.00"
                            name="cost_{nr_cost}_{i}" id="id_cost_{nr_cost}_{i}"
                            class="text-end text-basic-roow" {disabled}
        """
        if data:
            html += f"value='{data[f'cost_{i}']}' class='text-end'>"
        else:
            html += f">"
        """
                    </p>
                </td>
            </tr>
        """

    html += f"""
            <tr>
                <td class="bg-slategrey bx-slategrey px-15">
                    <p class="mt-2 mb-1 text-white">
    """
    html += _("total")
    html += f"""
                    </p>
                </td>
                <td class="px-15">
                    <p class="bb-dashed-slategrey mb-2 text-end">
                        € {"%.2f" % obj.tot_costs(nr_cost, False)}
                    </p>
                </td>
            </tr>

        </tbody>
    </table>
    """
    return mark_safe(html)


@register.filter(is_safe=True)
@register.simple_tag
def render_month_cost(kakebo: str, row: int = 5):
    html = "<tbody>"
    list_row = row + 1
    username, year, month = kakebo.split('-')
    # get User()
    user = User.objects.get(username=username)
    # get KakeboMonth()
    obj_month = KakeboMonth.objects.get(
        user=user,
        month=month,
        year=year,
    )
    obj_endmonth = KakeboEndOfMonthBalance.objects.get(month=obj_month)

    for i, obj in enumerate(KakeboWeek.objects.filter(user=user, month=obj_month), 1):
        data = []
        for table in KakeboWeekTable.objects.filter(kakebo=obj):
            data.append(table.display_total_table)
        val = sum(list(map(float, data)))

        html += "<tr>"
        if i == 1:
            html += f"""
                <td class="w-33 align-top text-end p-3" rowspan="{list_row + 1}">
                    <h5 class="text-lightseagreen">
                        spese settimali
                    </h5>
                </td>
            """

        css_value = ""
        if i == 1:
            css_value = "bt-lightseagreen"

        html += f"""
                <td class="bg-lightseagreen {css_value} px-15">
                    <p class="mt-2 mb-1 text-white">
                        week {i}
                    </p>
                </td>
                <td class="px-15 {css_value} bx-lightseagreen">
                    <p class="bb-dashed-lightseagreen mb-2 text-end">
                        {"%.2f" % val}
                    </p>
                </td>
        """
        html += "</tr>"

    obj = KakeboEndOfMonthBalance.objects.get(month=obj_month)
    html += f"""
        <tr>
            <td class="bg-lightseagreen bb-lightseagreen px-15">
                <p class="mt-2 mb-1 text-white">
    """
    html += _("utilities")
    html += f"""
                </p>
            </td>
            <td class="bb-lightseagreen bx-lightseagreen px-15">
                <p class="bb-dashed-lightseagreen mb-0 text-end">
                    € {obj.display_total_utilities}
                </p>
            </td>
        </tr>
    """

    # white space
    html += """
        </tbody>
        <tbody>
            <tr>
                <td>
                    <br>
                </td>
            </tr>
        </tbody>
    """

    html += """
        <tbody>
            <tr>
                <td></td>
                <td class="bg-lightseagreen px-15">
                    <p class="mt-2 mb-1 text-white">
    """
    html += _("total month cost")
    html += f"""
                    </p>
                </td>
                <td class="px-15">
                    <p class="bb-dashed-lightseagreen mb-2 text-end">
                        € {obj_endmonth.display_total_month}
                    </p>
                </td>
            </tr>
        </tbody>
    """
    return mark_safe(html)


@register.filter(is_safe=True)
@register.simple_tag
def render_answer_smile(answer: int):
    answer_list = [
        ("yes", "sunglasses"),
        ("almost", "expressionless"),
        ("no", "frown"),
    ]
    answer = answer_list[answer]

    html = f"""
    <button type="button" class="btn btn-primary" data-bs-toggle="modal" data-bs-target="#{answer[1]}Modal">
        <i class="bi bi-emoji-{answer[1]} h1"></i><br/>{answer[0]}
    </button>
    
    <div class="modal fade" id="{answer[1]}Modal" tabindex="-1" aria-labelledby="{answer[1]}ModalLabel" aria-hidden="true">
        <div class="modal-dialog">
            <div class="modal-content">
                <div class="modal-header">
                    <button type="button" class="btn-close" data-bs-dismiss="modal" aria-label="Close"></button>
                </div>
                <div class="modal-body">
                    <p>
                        Sei sicuro della tua decisione?
                    </p>
                    <p>
                        Cliccando invio chiuderai il mese e non sarà possibile effettuare
                        modifiche al mese corrente
                    </p>
                </div>
                <div class="modal-footer w-100">
                    <div class="container-fluid">
                        <div class="row">
                            <div class="col-6">
                                <button type="button" class="btn btn-secondary w-100" data-dismiss="modal">
                                    Torna indietro
                                </button>
                            </div>
                            <div class="col-6">
                                <button type="submit" class="btn btn-primary w-100" name="answer_{answer[0]}_check">
                                    Invia
                                </button>
                            </div>
                        </div>
                    </div>
                </div>
            </div>
        </div>
    </div>
    """
    return mark_safe(html)



from datetime import datetime

from django import template
from django.utils.safestring import mark_safe
from django.contrib.auth.models import User
from django.utils.translation import gettext_lazy as _
from ..constants import colors as list_colors
from ..models import KakeboMonth, KakeboWeek, KakeboWeekTable

register = template.Library()


def get_data_byobj(kakebo: dict, color: str, row: int, column: int):
    type_cost = int(list_colors.index(color))
    kakebo = KakeboWeekTable.objects.get(
        kakebo=kakebo, type_cost=type_cost
    ).data_row

    if not any(
            kakebo
            and kakebo.get(f"{column}", "")
            and kakebo.get(f"{column}", "").get(f"{row}", "")
    ):
        return None

    return kakebo[f"{column}"][f"{row}"]


@register.simple_tag
def white_space_table():
    # a simple body to render a white space
    html = """
    <tbody>
        <tr>
            <td>
                <br>
            </td>
        </tr>
    </tbody>
    """
    return mark_safe(html)


@register.filter(is_safe=True)
@register.simple_tag
def render_table(color: str = None, row: int = 7, name: str = None, kakebo: str = None, disabled: bool = False):
    """
    Render table with a determinate color and row
    """
    if color is None:
        color = "orange"

    # get kakebo table week from db
    username, year, week = kakebo.split('-')
    # get user
    user = User.objects.get(username=username)
    date_w = f"{year}-{week}-1"
    # get KakeboMonth()
    month = KakeboMonth.objects.get(
        user=user,
        month=datetime.strptime(date_w, "%Y-%W-%w").month,
        year=year,
    )
    obj = KakeboWeek.objects.get(
        user=user, month=month, week=week,
    )

    html = "<tbody>"
    for i in range(row):
        html += "<tr>"
        if i == 0:
            html += f"""
            <td class="align-top text-end p-3" rowspan="{row}">
                <h5 class="text-{color}">
                    {name}
                </h5>
            </td>
            """
        for clm in range(7):  # 7 days in one week
            if i == 0:
                html += f'<td class="bt-8-{color} bx-{color} pt-3 px-15">'
            elif i == row - 1:
                html += f'<td class="bb-{color} bx-{color} px-15">'
            else:
                html += f'<td class="bx-{color} px-15">'

            data_ = get_data_byobj(obj, color, i, clm)

            tag_name_modal = f"tag_name_{color}_{clm}_{i}"
            html += f"""
                <p class="w-100 mb-2 bb-dashed-{color} text-end">
            """

            if data_ is not None:
                html += f"{data_['desc'][:15]} => € {'%.2f' % data_['value']}"
            else:
                html += "<br />"

            if not disabled:
                html += f"""
                        <button type="button" class="btn" data-bs-toggle="modal" data-bs-target="#{tag_name_modal}">
                            <i class="bi bi-pencil-square"></i>
                        </button>
                    </p>
                """

            html += "</p></td>"

            if not disabled:
                html += f"""
                <div class="modal fade" id="{tag_name_modal}" tabindex="-1"
                    aria-labelledby="{tag_name_modal}_label" aria-hidden="true">
                    <div class="modal-dialog modal-lg modal-dialog-centered">
                        <div class="modal-content">
                            <div class="modal-header">
                                <h5 class="modal-title" id="{tag_name_modal}_label">
                                    Descrizione e valore
                                </h5>
                                <button type="button" class="btn-close" data-bs-dismiss="modal" aria-label="Close"></button>
                            </div>
                            <div class="modal-body">
                                    
                                <div class="form-group row">
                                    <div class="col-sm-10">
                                        <label class="col-form-label">
                                            Descrizione:
                                        </label>
                                        <textarea class="form-control" id="id_{tag_name_modal}_desc" 
                                            name="{tag_name_modal}_desc" rows="4" cols="50"
                """

                html += 'placeholder="description">'
                html += data_["desc"] if data_ is not None else ""
                html += "</textarea>"

                html += f"""
                                    </div>
                                    <div class="col-sm-2">
                                        <label class="col-form-label">
                                            Valore:
                                        </label>
                                        <input type="number" class="form-control" id="id_{tag_name_modal}_value"
                                            name="{tag_name_modal}_value" style="border: 1px solid grey"
                                            placeholder="0.00" min="0" step="0.01"
                """

                html += f"value={data_['value']}>" if data_ is not None else ">"

                html += """
                                    </div>
                                </div>
    
                            </div>
                            <div class="modal-footer">
                                <button type="submit" class="btn btn-primary">
                                    Salva
                                </button>
                            </div>
                        </div>
                    </div>
                </div>
                """
        # sidebar
        html += render_sidebar(obj, i, color, row - 1)
        # close row
        html += "</tr>"
    html += "</tbody>"
    return mark_safe(html)


@register.filter(is_safe=True)
@register.simple_tag
def render_table_total(totals: list, total_week: int):
    html = """
    <tbody>
        <tr>
            <td class="align-top text-end p-3" rowspan="{row}">
                <h5 class="text-slategrey">
    """
    html += _("total")
    html += """
                </h5>
            </td>
    """
    for value in totals:
        html += f"""
            <td class="b-slategrey px-15">
                <p class="bb-dashed-slategrey mb-0 text-end">
                    {value} €
                </p>
            </td>
        """
    html += f"""
        <td class="px-15">
            <p class="bg-slategrey mb-0 px-1 text-white">
                totale della settimana
            </p>
        </td>
        <td class="b-slategrey px-15">
            <p class="bb-dashed-slategrey mb-0 text-end">
                {total_week} €
            </p>
        </td>
    """
    html += """
        </tr>
    </tbody>
    """
    return mark_safe(html)


def render_sidebar(kakebo, row: int, color: str, total_row: int):
    # init cell html
    html = f"""
        <td class="m-3" width="7.4%">
    """

    type_cost = int(list_colors.index(color))
    obj = KakeboWeekTable.objects.get(
        kakebo=kakebo, type_cost=type_cost
    )

    # init and set value
    if row == total_row:
        val = (_("total"), obj.display_total_table)
        html += f'<p class="mx-3 p-1 bg-{color} text-white">'
    else:
        data = obj.get_list_sort_cost(total_row)
        if data and row < len(data):
            data = data[row]
            val = (data['desc'], data['value'])
        else:
            val = ("", "")
        html += '<p class="p-3">'

    html += f"""
            {val[0]}
            </p>
        </td>
        <td class="p-3" width="5%">
            <p class="bb-dashed-{color} mb-2 text-end">
             € {val[1]}
            </p>
        </td>
    """
    return html



from django import forms
from django.forms import formset_factory

from .models import KakeboEndOfMonthBalance
from .constants import colors


class KakeboWeekForm(forms.Form):
    def __init__(self, *args, **kwargs):
        super(KakeboWeekForm, self).__init__(*args, **kwargs)

        for color in colors:
            for row in range(9):  # casual max value (number)
                for clm in range(7):  # 7 days in one week
                    tag_name = f"tag_name_{color}_{clm}_{row}"
                    self.fields[f"{tag_name}_desc"] = forms.CharField(
                        max_length=200, required=False
                    )
                    self.fields[f"{tag_name}_value"] = forms.FloatField(
                        required=False
                    )


class SelectYearWeekForm(forms.Form):
    spare_cost = forms.CharField(widget=forms.Textarea(attrs={"rows": "5"}), required=False)
    target_reach = forms.CharField(widget=forms.Textarea(attrs={"rows": "5"}), required=False)
    spare = forms.FloatField(required=False)

    def __init__(self, *args, **kwargs):
        super(SelectYearWeekForm, self).__init__(*args, **kwargs)

        field_types = ["income", "outflow"]
        for field in field_types:
            self.fields[f"date_{field}"] = forms.DateField(
                required=False
            )
            self.fields[f"descr_{field}"] = forms.CharField(
                max_length=200, required=False
            )
            self.fields[f"value_{field}"] = forms.FloatField(
                required=False
            )


SelectYearWeekFormSet = formset_factory(SelectYearWeekForm, extra=2)


class EndOfMonthBalanceSheetForm(forms.Form):
    electricity = forms.FloatField(required=False)
    gas = forms.FloatField(required=False)
    tel_internet = forms.FloatField(required=False)
    water = forms.FloatField(required=False)
    waste = forms.FloatField(required=False)
    answer_1 = forms.CharField(widget=forms.Textarea(attrs={"rows": "5"}), required=False)
    answer_2 = forms.CharField(widget=forms.Textarea(attrs={"rows": "5"}), required=False)
    answer_3 = forms.CharField(widget=forms.Textarea(attrs={"rows": "5"}), required=False)

    def __init__(self, *args, **kwargs):
        conclusion = kwargs['initial']['conclusion']
        super(EndOfMonthBalanceSheetForm, self).__init__(*args, **kwargs)

        for j in range(2):
            self.fields[f"cost_{j}_name"] = forms.CharField(max_length=200, required=False)
            for i in range(1, 6):
                self.fields[f"cost_{j}_{i}"] = forms.FloatField(required=False)

        for field in self.fields:
            self.fields[field].widget.attrs.update({"class": "form-control"})
            if conclusion and field != 'answer_3':
                self.fields[field].widget.attrs['disabled'] = 'disabled'



from calendar import monthcalendar, month_name
from datetime import datetime, date, timedelta

from django.urls import reverse_lazy
from django.http import HttpResponseRedirect
from django.views.generic import TemplateView, FormView
from django.contrib.auth.mixins import LoginRequiredMixin

from ..models import KakeboMonth, KakeboWeek, KakeboWeekTable, KakeboEndOfMonthBalance
from ..forms import SelectYearWeekFormSet, KakeboWeekForm, EndOfMonthBalanceSheetForm
from ..constants import colors
from ..utils import find_indices


class Index(TemplateView):
    template_name = "basic/home/index.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["five_reason"] = [
            {
                "name": "ORDINE",
                "description": "Lorem ipsium",
                "favicon": '<i class="bi bi-list-ul"></i>',
            },
            {
                "name": "CONTROLLO",
                "description": "Lorem ipsium",
                "favicon": '<i class="bi bi-controller"></i>',
            },
            {
                "name": "RISPARMIO",
                "description": "Lorem ipsium",
                "favicon": '<i class="bi bi-piggy-bank"></i>',
            },
            {
                "name": "AUTODISCIPLINA",
                "description": "Lorem ipsium",
                "favicon": '<i class="bi bi-cup-hot"></i>',
            },
            {
                "name": "SERENITA'",
                "description": "Lorem ipsium",
                "favicon": '<i class="bi bi-emoji-laughing"></i>',
            },
        ]
        return context


class SelectYearWeekFormView(LoginRequiredMixin, FormView):
    form_class = SelectYearWeekFormSet
    template_name = "basic/kakebo/calendar.html"

    def dispatch(self, request, *args, **kwargs):
        if not request.user.is_authenticated:
            return self.handle_no_permission()

        # set month and year in kwargs if they aren't exist
        if not self.kwargs.get("month"):
            self.kwargs["month"] = date.today().month
        if not self.kwargs.get("year"):
            self.kwargs["year"] = date.today().year

        # init obj in self
        self.obj, _ = KakeboMonth.objects.get_or_create(
            user=self.request.user,
            month=self.kwargs['month'],
            year=self.kwargs['year'],
        )

        obj_reports = KakeboEndOfMonthBalance.objects.filter(month=self.obj)
        if obj_reports:
            self.obj_report = obj_reports[0]
            self.conclusion = self.obj_report.conclusion
        else:
            self.obj_report, self.conclusion = None, None

        return super().dispatch(request, *args, **kwargs)

    def form_valid(self, form):
        if not self.obj_report.conclusion:
            budget = {}
            for i, row in enumerate(form.cleaned_data):
                budget[f'_income_{i}'] = {}
                budget[f'_outflow_{i}'] = {}
                for key, value in row.items():
                    if '_income' in key:
                        if isinstance(value, (datetime, date)):
                            value = value.strftime("%Y-%m-%d")
                        value = "" if value is None else value
                        budget[f'_income_{i}'][key] = value
                    if '_outflow' in key and value:
                        if isinstance(value, (datetime, date)):
                            value = value.strftime("%Y-%m-%d")
                        budget[f'_outflow_{i}'][key] = value
            self.obj.budget = budget

            cd_0 = form.cleaned_data[0]
            if cd_0:
                self.obj.spare_cost = cd_0.get("spare_cost", "")
                self.obj.target_reach = cd_0.get("target_reach", "")
                self.obj.spare = cd_0.get("spare") if cd_0.get("spare") else 0

            self.obj.save()
        return self.get_success_url()

    def get_success_url(self):
        return HttpResponseRedirect(
            reverse_lazy(
                "kakebo-calendar", kwargs={
                    "year": self.kwargs["year"],
                    "month": self.kwargs["month"],
                },
            )
        )

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        month, year = self.kwargs['month'], self.kwargs['year']

        context["day_list"] = [
            "Monday",
            "Tuesday",
            "Wednesday",
            "Thursday",
            "Friday",
            "Saturday",
            "Sunday",
        ]
        context["week_list"] = self.get_week_dict(
            monthcalendar(year, month)
        )

        context["month"] = f"{month_name[month]} {year}"
        context['key_kakebo'] = f'{self.request.user.username}-{self.kwargs["month"]} - {self.kwargs["year"]}'
        context["income"], context["outflow"] = self.obj.display_totals_budget
        context["spare_cost"] = self.obj.spare_cost
        context["target_reach"] = self.obj.target_reach
        context["spare"] = "%.2f" % self.obj.spare
        available_money = self.obj.display_available_money
        context["available_money"] = "%.2f" % available_money if available_money else None
        context['disabled'] = "disabled" if self.conclusion else ""
        return context

    def get_week_dict(self, week_list) -> dict:
        nr_week = self.get_nr_week(week_list)
        for week in week_list:
            for i, day in enumerate(week):
                week[i] = ("", day)

        # set first week
        week = week_list[0]
        if ("", 0) in week:
            day = 31
            month = self.kwargs['month'] - 1
            for i in sorted(find_indices(week, ("", 0)), reverse=True):
                week[i] = ("another-month", day, month)
                day -= 1

        # set last week
        week = week_list[-1]
        if ("", 0) in week:
            day = 1
            month = self.kwargs['month'] + 1
            for i in find_indices(week, ("", 0)):
                week[i] = ("another-month", day, month)
                day += 1

        week_dict = {}
        for i, week in enumerate(week_list):
            week_dict[nr_week[i]] = week

        return week_dict

    def get_nr_week(self, week_list):
        nr_week = []
        for week in week_list:
            day = [x for x in week if x][0]
            nr_week.append(date(self.kwargs['year'], self.kwargs['month'], day).isocalendar().week)
        return nr_week


class KakeboWeekFormView(LoginRequiredMixin, FormView):
    form_class = KakeboWeekForm
    template_name = "basic/kakebo/week.html"

    def dispatch(self, request, *args, **kwargs):
        if not request.user.is_authenticated:
            return self.handle_no_permission()

        # init year and week in self
        self.year = kwargs['year']
        self.week = kwargs['week']

        # get month by year and week
        date_w = f"{self.year}-{self.week}-1"
        month = datetime.strptime(date_w, "%Y-%W-%w").month
        # get KakeboMonth()
        self.obj_month, _ = KakeboMonth.objects.get_or_create(
            user=self.request.user,
            month=month,
            year=self.kwargs['year'],
        )

        # init obj with relative tables (=> tb_{color}) in self
        self.obj, _ = KakeboWeek.objects.get_or_create(
            user=self.request.user,
            month=self.obj_month,
            week=self.week,
        )
        for i, color in enumerate(colors):
            table, _ = KakeboWeekTable.objects.get_or_create(
                kakebo=self.obj, type_cost=i
            )
            setattr(self, f'tb_{color}', table)

        # init date from week and year param in self.time_
        self.time_ = self.obj.display_start_week

        obj_reports = KakeboEndOfMonthBalance.objects.filter(month=self.obj_month)
        if obj_reports:
            self.obj_report = obj_reports[0]
            self.conclusion = self.obj_report.conclusion
        else:
            self.obj_report, self.conclusion = None, None

        return super().dispatch(request, *args, **kwargs)

    def form_valid(self, form):
        if self.conclusion is not None:
            cd = form.cleaned_data
            for color in colors:
                data = {}
                for clm, day in enumerate(self.get_list_day(self.time_)):
                    data[clm] = {}
                    for row in range(9):
                        tag_name = f"tag_name_{color}_{clm}_{row}"
                        desc = cd.get(f"{tag_name}_desc", None)
                        value = cd.get(f"{tag_name}_value", None)
                        if desc or value:
                            data[clm][row] = {"desc": desc, "value": value}
                obj = getattr(self, f'tb_{color}')
                obj.data_row = data
                obj.save()
        return self.get_success_url()

    def get_success_url(self):
        return HttpResponseRedirect(
            reverse_lazy(
                "kakebo-week", kwargs={
                    "year": self.kwargs["year"],
                    "week": self.kwargs["week"],
                },
            )
        )

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        list_tr = []
        list_type = KakeboWeekTable.LIST_TYPE
        # TODO: add management in back-office view
        for i, lenrows in enumerate([10, 7, 7, 7]):
            list_tr.append((colors[i], lenrows, list_type[i][1]))
        context['list_tr'] = list_tr

        context['cell_'] = self.get_list_day(self.time_)
        context['row_'] = [i for i in range(7)]
        context['key_kakebo'] = f'{self.request.user.username}-{self.kwargs["year"]}-{self.kwargs["week"]}'

        totals_days = []
        for i in range(7):
            totals = []
            for color in colors:
                obj = getattr(self, f'tb_{color}')
                totals.append(obj.total_column(i))
            totals_days.append('%.2f' % (sum(totals)))

        context["totals_days"] = totals_days
        context["total_week"] = sum(list(map(float, totals_days)))
        context['disabled'] = "disabled" if self.conclusion else ""
        return context

    @staticmethod
    def get_list_day(today: datetime) -> list:
        list_days = []
        for i in range(7):
            data__ = f"{today.strftime('%A')} - {today.strftime('%d')}"
            list_days.append(data__)
            today += timedelta(days=1)
        return list_days


class EndOfMonthBalanceSheetFormView(LoginRequiredMixin, FormView):
    form_class = EndOfMonthBalanceSheetForm
    template_name = "basic/kakebo/month-balance.html"

    def dispatch(self, request, *args, **kwargs):
        if not request.user.is_authenticated:
            return self.handle_no_permission()

        user = self.request.user
        self.obj_month = KakeboMonth.objects.get(
            user=user,
            month=self.kwargs["month"],
            year=self.kwargs["year"],
        )
        self.obj_list = KakeboWeek.objects.filter(user=user, month=self.obj_month)
        self.obj, _ = KakeboEndOfMonthBalance.objects.get_or_create(
            month=self.obj_month
        )

        return super().dispatch(request, *args, **kwargs)

    def form_valid(self, form):
        cd = form.cleaned_data

        if not self.obj.conclusion:
            self.obj.electricity = cd['electricity'] if cd['electricity'] else 0
            self.obj.gas = cd['gas'] if cd['gas'] else 0
            self.obj.tel_internet = cd['tel_internet'] if cd['tel_internet'] else 0
            self.obj.water = cd['water'] if cd['water'] else 0
            self.obj.waste = cd['waste'] if cd['waste'] else 0
            self.obj.answer_1 = cd['answer_1']
            self.obj.answer_2 = cd['answer_2']

            costs = {}
            for j in range(2):
                key_name = cd[f"cost_{j}_name"]
                if key_name:
                    costs[key_name] = {}
                    for i in range(1, 6):
                        costs[key_name][f"cost_{i}"] = cd[f"cost_{j}_{i}"]
            self.obj.costs_data = costs

            # save KakeboEndOfMonthBalance()
            self.obj.save()

            if "answer_yes_check" in self.request.POST:
                self.obj.conclusion = 1
            if "answer_almost_check" in self.request.POST:
                self.obj.conclusion = 2
            if "answer_no_check" in self.request.POST:
                self.obj.conclusion = 3
            self.obj.save()

        self.obj.answer_3 = cd['answer_3']
        self.obj.save()

        return self.get_success_url()

    def get_success_url(self):
        return HttpResponseRedirect(
            reverse_lazy(
                "kakebo-balance", kwargs={
                    "year": self.kwargs["year"],
                    "month": self.kwargs["month"],
                },
            )
        )

    def get_initial(self, **kwargs):
        initial = {
            "conclusion": self.obj.conclusion,
            "answer_1": self.obj.answer_1,
            "answer_2": self.obj.answer_2,
            "answer_3": self.obj.answer_3,
        }
        return initial

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['list_colors'] = colors
        context['key_kakebo'] = f'{self.request.user.username}-{self.kwargs["year"]}-{self.kwargs["month"]}'
        list_utilities = ["electricity", "gas", "tel_internet", "water", "waste"]
        context['list_utilities'] = [(x, getattr(self.obj, f"display_{x}")) for x in list_utilities]
        context['tot_utilities'] = self.obj.display_total_utilities
        context['len_week'] = len(self.obj_list)
        context['tot_available'] = self.obj_month.display_available_money
        context['tot_costs'] = self.obj.display_total_month
        context['diff_available_costs'] = self.obj.display_diff_available_costs
        context['obj_conclusion'] = self.obj.get_conclusion_display
        context['disabled'] = "disabled" if self.obj.conclusion else ""
        return context



import requests

from django.conf import settings
from django.http import HttpResponseRedirect
from django.urls import reverse_lazy
from django.views.generic import FormView
from django.contrib import messages
from django.contrib.auth.models import User
from django.contrib.auth.views import (
    LoginView,
    PasswordResetView,
    PasswordResetDoneView,
    PasswordResetConfirmView,
)

from .forms import RegisterForm, AuthForm


class RegisterPageFormView(FormView):
    form_class = RegisterForm
    template_name = "user/sign-up.html"

    def dispatch(self, *args, **kwargs):
        if self.request.user.is_authenticated:
            return self.get_success_url()
        return super().dispatch(*args, **kwargs)

    def form_valid(self, form):
        request = self.request
        cd = form.cleaned_data

        username = cd["username"]
        email = cd["email"]
        password = cd["password"]

        if getattr(settings, 'GOOGLE_RECAPTCHA_SECRET_KEY', None) is not None:
            """ Start reCAPTCHA validation """
            res = request.POST.get("g-recaptcha-response")
            url = "https://www.google.com/recaptcha/api/siteverify"
            values = {
                "secret": settings.GOOGLE_RECAPTCHA_SECRET_KEY,
                "response": res,
            }
            response = requests.post(url=url, data=values)
            result = response.json()
            """ End reCAPTCHA validation """

            if not result.get("success"):
                form.add_error(field=None, error="Verifica Captcha fallita")
                return self.form_invalid(form)

        User.objects.create_user(
            username, email, password
        )
        messages.success(request, "Utente creato con successo, controlla la mail per confermarla")
        return self.get_success_url()

    def get_success_url(self):
        return HttpResponseRedirect(reverse_lazy("login"))


def active_user_mail():
    pass


class AccountLoginView(LoginView):
    """
    Display the custom login form and handle the login action.
    """

    form_class = AuthForm
    template_name = "user/sign-in.html"
    success_url = reverse_lazy("index")


class AccountPasswordResetView(PasswordResetView):
    pass


class AccountPasswordResetDoneView(PasswordResetDoneView):
    pass


class AccountPasswordResetConfirmView(PasswordResetConfirmView):
    pass


class AccountPasswordResetCompleteView(PasswordResetDoneView):
    pass



from django import forms
from django.utils.translation import gettext_lazy as _
from django.contrib.auth.models import User
from django.contrib.auth.forms import AuthenticationForm


class RegisterForm(forms.Form):
    username = forms.CharField(
        label=_("Username"),
        widget=forms.TextInput(),
    )
    email = forms.EmailField(
        label=_("Email"),
        widget=forms.EmailInput(),
    )
    password = forms.CharField(
        label=_("Password"),
        widget=forms.PasswordInput(),
    )
    password2 = forms.CharField(
        label=_("Confirm password"),
        widget=forms.PasswordInput(),
    )
    condition_check = forms.BooleanField(
        required=False
    )

    def __init__(self, *args, **kwargs):
        super(RegisterForm, self).__init__(*args, **kwargs)

        for field in self.fields:
            self.fields[field].widget.attrs.update(
                {
                    "class": "form-control",
                    "placeholder": self.fields[field].label,
                }
            )

    def clean(self):
        data = self.cleaned_data
        password = data.get("password")
        password2 = data.get("password2")
        if password2 != password:
            raise forms.ValidationError(_("Passwords aren't the same"))

        condition_check = data.get("condition_check")
        if condition_check is False:
            raise forms.ValidationError(_("You must accept the conditions of sale"))
        return data

    def clean_email(self):
        email = self.cleaned_data.get("email")
        qs = User.objects.filter(email=email)
        if qs.exists():
            raise forms.ValidationError(_("Email already in use"))
        return email


class AuthForm(AuthenticationForm):
    def __init__(self, *args, **kwargs):
        super(AuthForm, self).__init__(*args, **kwargs)

        for field in self.fields:
            self.fields[field].widget.attrs.update(
                {
                    "class": "form-control",
                    "placeholder": str(field).capitalize(),
                }
            )



from django.conf import settings
from django.db import migrations, models
from django.db.models.deletion import CASCADE


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="KakeboMonth",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("month", models.PositiveSmallIntegerField(default=1)),
                ("year", models.PositiveSmallIntegerField(default=1980)),
                ("budget", models.JSONField(default=dict)),
                ("spare_cost", models.TextField(blank=True, default="", null=True)),
                ("target_reach", models.TextField(blank=True, default="", null=True)),
                ("spare", models.FloatField(blank=True, default=0, null=True)),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "Kakebo Month",
            },
        ),
        migrations.CreateModel(
            name="KakeboWeek",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("week", models.IntegerField(default=1)),
                (
                    "month",
                    models.ForeignKey(
                        on_delete=CASCADE,
                        to="django_kakebo.kakebomonth",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "Kakebo Week",
            },
        ),
        migrations.CreateModel(
            name="UtilitiesCost",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=150)),
            ],
        ),
        migrations.CreateModel(
            name="KakeboWeekTable",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("data_row", models.JSONField(default=dict)),
                (
                    "type_cost",
                    models.PositiveSmallIntegerField(
                        choices=[
                            (0, "basic necessities"),
                            (1, "optional"),
                            (2, "culture and leisure"),
                            (3, "extras and unexpected"),
                        ],
                        default=0,
                    ),
                ),
                (
                    "kakebo",
                    models.ForeignKey(
                        on_delete=CASCADE,
                        to="django_kakebo.kakeboweek",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="KakeboEndOfMonthBalance",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("electricity", models.FloatField(default=0)),
                ("gas", models.FloatField(default=0)),
                ("tel_internet", models.FloatField(default=0)),
                ("water", models.FloatField(default=0)),
                ("waste", models.FloatField(default=0)),
                ("costs_data", models.JSONField(default=dict)),
                ("answer_1", models.TextField(blank=True, default="", null=True)),
                ("answer_2", models.TextField(blank=True, default="", null=True)),
                ("answer_3", models.TextField(blank=True, default="", null=True)),
                (
                    "conclusion",
                    models.PositiveSmallIntegerField(
                        choices=[(0, ""), (1, "yes"), (2, "almost"), (3, "no")],
                        default=0,
                    ),
                ),
                (
                    "month",
                    models.ForeignKey(
                        on_delete=CASCADE,
                        to="django_kakebo.kakebomonth",
                    ),
                ),
            ],
        ),
    ]



from django import template
from django.contrib.auth.models import User
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _
from ..models import KakeboMonth

register = template.Library()


@register.filter(is_safe=True)
@register.simple_tag
def render_month_budget(kakebo: str, section: str, nr_row: int = 4, disabled: bool = False):
    username, month, year = kakebo.split('-')
    user = User.objects.get(username=username)
    obj = KakeboMonth.objects.get(
        user=user, month=month, year=year
    )
    budget = obj.budget
    disabled = "disabled" if disabled else ""

    html = ""
    for i in range(nr_row):
        value = budget.get(f"_{section}_{i}")
        val_date, val_descr, val_value = "", "", ""
        if value:
            val_date = value.get(f"date_{section}")
            val_descr = value.get(f"descr_{section}")
            val_value = "%.2f" % value.get(f"value_{section}")

        html += f"""
        <tr>
            <td></td>
            <td class="bs-10-white">
                <p class="bb-dashed-slategrey w-100 pt-3">
                    <input name="form-{i}-date_{section}" id="id_form-{i}-date_{section}"
                        type="date" value="{val_date}" {disabled}>
                </p>
            </td>
            <td class="bs-10-white">
                <p class="bb-dashed-slategrey w-100 pt-3">
                    <input name="form-{i}-descr_{section}" id="id_form-{i}-descr_{section}"
                    value="{val_descr}" {disabled}>
                </p>
            </td>
            <td class="bs-10-white">
                <p class="bb-dashed-slategrey w-100 pt-3">
                    <input name="form-{i}-value_{section}" id="id_form-{i}-value_{section}"
                    value="{val_value}" {disabled}>
                </p>
            </td>
        </tr>
        """

    total_ = [
        (_("total income"), "cyan", obj.display_totals_budget[0]),
        (_("total steady outflow"), "blue", obj.display_totals_budget[1])
    ]
    total_ = total_[0] if section == 'income' else total_[1]
    html += f"""
        <tr>
            <td></td>
            <td colspan="2" class="bs-10-white">
                <p class="w-100 pt-3 ps-3 bg-{total_[1]}-basic text-white">
                    {total_[0]}
                </p>
            </td>
            <td class="bs-10-white">
                <p class="bb-dashed-slategrey w-100 pt-3">
                    € {total_[2]}
                </p>
            </td>
        </tr>
    """
    return mark_safe(html)



import os
import glob
import setuptools

__version__ = "0.1.1"

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()


def get_install_requires() -> list:
    """Returns requirements.txt parsed to a list"""
    fname = os.path.join(os.path.dirname(__file__), "requirements.txt")
    targets = []
    if os.path.exists(fname):
        with open(fname, 'r') as f:
            targets = f.read().splitlines()
    return targets


setuptools.setup(
    name="django_kakebo",
    version=__version__,
    author="shadMod",
    author_email="support@shadmod.it",
    description="A simple site made with django and exploiting the kakebo method",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/shadMod/django-kakebo/",
    download_url=f"https://github.com/shadMod/django-kakebo/archive/refs/tags/{__version__}.tar.gz",
    project_urls={
        "Bug Tracker": "https://github.com/shadMod/django-kakebo/issues/",
    },
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
    include_package_data=True,
    packages=[
        "django_kakebo", "django_kakebo.migrations", "django_kakebo.src.django_kakebo",
        "django_kakebo.src.django_kakebo.migrations", "django_kakebo.src.django_kakebo.templatetags",
        "django_kakebo.src.django_kakebo.views", "django_kakebo.src.user"
    ],
    data_files=[
        (
            "static",
            [fn for fn in glob.iglob("django_kakebo/src/django_kakebo/static/**/*", recursive=True) if "." in fn],
        ),
        (
            "themes",
            [fn for fn in glob.iglob("django_kakebo/src/themes/**/*", recursive=True) if "." in fn],
        ),
        (
            "static_user",
            [fn for fn in glob.iglob("django_kakebo/src/user/static/**/*", recursive=True) if "." in fn],
        ),
    ],
    install_requires=get_install_requires(),
    python_requires=">=3.8",
)
